# open this flag when support more package about go in the futrue
%bcond_with fish

%global libnetwork_commit dcdf8f176d1e13ad719e913e796fb698d846de98
%global proxy_builddir libnetwork-%{libnetwork_commit}
%global tini_version 0.19.0
%global __bindir %_builddir/%{name}-%{version}
%global goipath github.com/docker

Summary:       The open-source application container engine
Name:          moby
Version:       20.10.21
Release:       6%{?dist}
License:       ASL 2.0
URL:           https://mobyproject.org
Source0:       https://github.com/%{name}/%{name}/archive/refs/tags/v%{version}.tar.gz
# note: ignore "#/cli-20.10.21.tar.gz"
Source1:       https://github.com/docker/cli/archive/refs/tags/v%{version}.tar.gz#/cli-20.10.21.tar.gz
# note: ignore "#/libnetwork.zip"
Source2:       https://github.com/moby/libnetwork/archive/%{libnetwork_commit}.zip#/libnetwork.zip
Source3:       https://github.com/krallin/tini/archive/refs/tags/v%{tini_version}.tar.gz
Source4:       docker.sysconfig
Source5:       moby-engine-systemd-sysusers.conf

BuildRequires: pkgconfig(systemd) golang >= 1.19-1 btrfs-progs-devel device-mapper-devel glibc-static
BuildRequires: libselinux-devel libtool-ltdl-devel pkgconfig selinux-policy selinux-policy-devel sqlite-devel
BuildRequires: systemd-devel libseccomp-devel tar git cmake firewalld-filesystem go-md2man go-rpm-macros

Requires: container-selinux containerd runc systemd pigz iptables libcgroup tar xz
Requires: systemd-units shadow-utils systemd-rpm-macros device-mapper-libs
Provides: docker = %{version}-%{release}
Provides: docker-ce = %{version}-%{release}
Conflicts: podman-docker

%description
Docker is an open source project to build, ship and run any application as a
lightweight container.

Docker containers are both hardware-agnostic and platform-agnostic. This means
they can run anywhere, from your laptop to the largest EC2 compute instance and
everything in between - and they don't require you to use a particular
language, framework or packaging system. That makes them great building blocks
for deploying and scaling web apps, databases, and backend services without
depending on a particular stack or provider.

%if %{with fish}
%package fish-completion
Summary:       Fish completion files for %{name}
BuildArch:     noarch
Requires:      %{name} = %{version}-%{release}
Requires:      fish
Provides:      docker-fish-completion = %{version}-%{release}

%description  fish-completion
This package installs the open-source application container engine.
%endif

%package zsh-completion
Summary:       Zsh completion files for %{name}
BuildArch:     noarch
Requires:      %{name} = %{version}-%{release}
Requires:      zsh
Provides:      docker-zsh-completion = %{version}-%{release}

%description   zsh-completion
This package installs the open-source application container engine.

%package nano
Summary:       GNU nano syntax highlighting files for Moby
BuildArch:     noarch
Requires:      %{name} = %{version}-%{release}
Requires:      nano

%description   nano
This package installs the open-source application container engine.

%prep
# fix -a [num] fail
%setup -q -b1 -b2 -b3 -n %{name}-%{version}
mv ../cli-%{version} .
mv ../libnetwork-%{libnetwork_commit} .
mv ../tini-%{tini_version} .

git init -q
git config user.name rpm-build
git config user.email '<rpm-build>'
git add .
git commit -q --allow-empty -a --author 'rpm-build <rpm-build>' -m '%{name}-%{version} base'

%build
mkdir -p %{__bindir}

# build docker-proxy
pushd %{proxy_builddir}
    export GOPATH="$PWD/.gopath"
    mkdir -p $GOPATH/src/%{goipath}
    ln -sfn $PWD $GOPATH/src/%{goipath}/libnetwork
    %gobuild -o %{__bindir}/docker-proxy %{goipath}/libnetwork/cmd/proxy
popd

# build docker-init
pushd tini-%{tini_version}
    %cmake
    make tini-static -C "%{__cmake_builddir}"
popd

# build docker engine
(
    export GOPATH="$PWD/.gopath"
    mkdir -p $GOPATH/src/%{goipath}
    ln -sfn $PWD $GOPATH/src/%{goipath}/docker
    export LDFLAGS="-w"
    export LDFLAGS+=" -X github.com/docker/docker/dockerversion.Version=%{version}"
    export LDFLAGS+=" -X github.com/docker/docker/dockerversion.IAmStatic=false"
    export DOCKER_BUILDTAGS="seccomp selinux journald"
    export BUILDTAGS="${DOCKER_BUILDTAGS}"
    export GOBUILDTAGS="${BUILDTAGS}"
    %gobuild -o %{__bindir}/dockerd %{goipath}/docker/cmd/dockerd
)

# buid docker cli
pushd cli-%{version}
    export GOPATH="$PWD/.gopath"
    mkdir -p $GOPATH/src/%{goipath}
    ln -sfn $PWD $GOPATH/src/%{goipath}/cli
    export LDFLAGS="-w"
    export LDFLAGS+=" -X github.com/docker/cli/cli/version.Version=%{version}"
    export BUILDTAGS="pkcs11"
    export GOBUILDTAGS="${BUILDTAGS}"
    %gobuild -o %{__bindir}/docker %{goipath}/cli/cmd/docker
    man/md2man-all.sh
popd

%install
# install binary
install -Dpm 755 %{__bindir}/docker %{__bindir}/dockerd -t %{buildroot}%{_bindir}/

# install proxy
install -m0755 %{__bindir}/docker-proxy %{buildroot}/%{_bindir}/

# install docker-init
install -Dpm 755 tini-%{tini_version}/%{__cmake_builddir}/tini-static %{buildroot}%{_libexecdir}/docker/docker-init

# install udev rules
install -Dpm 644 contrib/udev/80-docker.rules -t %{buildroot}%{_usr}/lib/udev/rules.d/

# add init scripts
install -Dpm 644 contrib/init/systemd/docker.socket -t %{buildroot}%{_unitdir}/
install -p -m 644 contrib/init/systemd/docker.service %{buildroot}/%{_unitdir}/docker.service

# for additional args
install -Dpm 644 %{SOURCE4} %{buildroot}%{_sysconfdir}/sysconfig/docker

# Install sysusers configuration
install -Dpm 644 %{SOURCE5} %{buildroot}%{_sysusersdir}/moby-engine.conf

# add bash, zsh, and fish completions
install -Dpm 644 cli-%{version}/contrib/completion/bash/docker -t %{buildroot}%{_datadir}/bash-completion/completions/
install -Dpm 644 cli-%{version}/contrib/completion/zsh/_docker -t %{buildroot}%{_datadir}/zsh/site-functions/
%if %{with fish}
install -Dpm 644 cli-%{version}/contrib/completion/fish/docker.fish -t %{buildroot}%{_datadir}/fish/vendor_completions.d/
%endif

# add nano
install -d %{buildroot}/usr/share/nano
install -p -m 644 contrib/syntax/nano/Dockerfile.nanorc %{buildroot}/usr/share/nano/Dockerfile.nanorc

# install manpages
install -Dpm 644 cli-%{version}/man/man1/*.1 -t %{buildroot}%{_mandir}/man1/
install -Dpm 644 cli-%{version}/man/man5/*.5 -t %{buildroot}%{_mandir}/man5/
install -Dpm 644 cli-%{version}/man/man8/*.8 -t %{buildroot}%{_mandir}/man8/

%pre
%{_rpmconfigdir}/sysusers.generate-pre.sh %{SOURCE5}

%post
%systemd_post docker.service docker.socket
%firewalld_reload

%preun
%systemd_preun docker.service docker.socket

%postun
%systemd_postun_with_restart docker.service

%files
%license LICENSE
%doc AUTHORS CHANGELOG.md CONTRIBUTING.md MAINTAINERS NOTICE README.md
%config(noreplace) %{_sysconfdir}/sysconfig/docker
%{_bindir}/docker
%{_bindir}/dockerd
%{_bindir}/docker-proxy
%dir %{_libexecdir}/docker/
%{_libexecdir}/docker/docker-init
%{_usr}/lib/udev/rules.d/80-docker.rules
%{_unitdir}/docker.service
%{_unitdir}/docker.socket
%{_sysusersdir}/moby-engine.conf
%{_datadir}/bash-completion/completions/docker
%{_mandir}/man1/docker*.1*
%{_mandir}/man5/{Dockerfile,docker-config-json}.5*
%{_mandir}/man8/dockerd.8*

%files zsh-completion
%{_datadir}/zsh/site-functions/_docker

%if %{with fish}
%files fish-completion
%{_datadir}/fish/vendor_completions.d/docker.fish
%endif

%files nano
%dir %{_datadir}/nano
%{_datadir}/nano/Dockerfile.nanorc

%changelog
* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 20.10.21-6
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Apr 07 2023 rockerzhu <rockerzhu@tencent.com> - 20.10.21-5
- Condflicts with podman.

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 20.10.21-4
- Rebuilt for OpenCloudOS Stream 23

* Fri Dec 16 2022 Shuo Wang <abushwang@tencent.com> - 20.10.21-3
- apply %gobuild macro

* Fri Dec 16 2022 Shuo Wang <abushwang@tencent.com> - 20.10.21-2
- rename docker to moby & enable man

* Thu Dec 15 2022 Shuo Wang <abushwang@tencent.com> - 20.10.21-1
- initial build
